from models.model import ClientModel
import redis

class RedisClient(ClientModel):
    def __init__(self, host, port, db, charset="utf-8", decode_responses=True, *args, **kwargs):
        self.host=host
        self.port=port
        self.db=db
        self.charset=charset
        self.decode_responses=decode_responses
        self.redis=redis.Redis(
            host=self.host,
            port=self.port,
            db=self.db,
            charset=self.charset,
            decode_responses=self.decode_responses
        )
        super().__init__(*args, **kwargs)

    def set(
        self,
        arrayName:str,
        key:str,
        value:str
    ):
        self.redis.hset(arrayName, key, value)

    def get(self, arrayName:str) -> list:
        return self.redis.hgetall(arrayName)

    def _getRedisClient(self) -> redis.Redis:
        return self.redis

    def getHost(self) -> str:
        return self.host



from abc import ABC, abstractmethod
from ...utils.util import validatePluginData, guessMetricByType

class ClientModel(ABC):
    '''
        date_created is a string in the format of MMM DDDD, YYYY
            example: Aug 18th, 2022
        match_list is a list of strings that are used to match the keys in the dict
            example: ['Fabletics', 'Yitty']
    '''
    def __init__(self, _dict=None, author=None, plugin_version=None, heartbeat_required=None, date_created=None, match_list=None):
        self.dict = {}
        self.author = author
        self.__setitem__('author', self.author)
        self.plugin_version = plugin_version
        self.__setitem__('plugin_version', self.plugin_version)
        self.heartbeat_required = heartbeat_required
        self.__setitem__('heartbeat_required', self.plugin_version)
        self.date_created = date_created
        self.__setitem__('date_created', self.date_created)
        if match_list:
            for key in _dict.keys():
                for i in match_list:
                    if i.lower() in key.lower():
                        self.__setitem__(key, _dict[key])
        else:
            self.dict.update(_dict)

    def __getitem__(self, key):
        return self.dict[key]
    def __setitem__(self, key, value):
        self.dict[key] = value
    def __delitem__(self, key):
        del self.dict[key]
    def __contains__(self, key):
        return key in self.dict
    def __len__(self):
        return len(self.dict)
    def __iter__(self):
        return iter(self.dict)
    def __legthCheck__(self, str:str):
        # string limit is 20 on site24x7 plugin dashboard
        if len(str) > 20:
            raise Exception("String length is greater than 20")

    def setAuthor(self, author:str):
        self.__legthCheck__(author)
        self.author = author
        self.__setitem__('author', self.author)

    def setPluginVersion(self, plugin_version:int):
        self.__legthCheck__(plugin_version)
        self.plugin_version = plugin_version
        self.__setitem__('plugin_version', self.plugin_version)

    def setHeartbeatRequired(self, heartbeat_required:bool):
        self.__legthCheck__(heartbeat_required)
        self.heartbeat_required = heartbeat_required
        self.__setitem__('heartbeat_required', self.heartbeat_required)
    
    def setDateCreated(self, date_created:str):
        self.__legthCheck__(date_created)
        self.date_created = date_created
        self.__setitem__('date_created', self.date_created)

    def validate(self):
        validatePluginData(self.dict)
    
    def set_metric_types(self):
        self.validate()
        metric_units = guessMetricByType({}, self.dict)
        self.__setitem__('units', metric_units)



#!/usr/bin/env python3
FUNCTIONAL_KEYS = 'plugin_version|heartbeat_required|status|units|msg|onchange|display_name|AllAttributeChart'
def guessMetricByType(main_dict, check_dict):
    '''only accepts int, float and str'''
    for i in check_dict:
        if i in FUNCTIONAL_KEYS:
            continue
        if isinstance(check_dict[i], tuple):
            continue
        elif isinstance(check_dict[i], list):
            continue
        elif isinstance(check_dict[i], dict):
            continue
        try:
            float(check_dict[i])
            main_dict[i] = 'count'
        except ValueError:
            main_dict[i] = 'string'
    return main_dict

def validatePluginData(result:dict):
    '''validation method - This is only for Output validation purpose'''
    mandatory = ['heartbeat_required','plugin_version']
    value = {'heartbeat_required':["true","false",True,False],'status':[0,1]}
    for field in mandatory:
        if field not in result:
            raise Exception(f"{field} is mandatory")
    for field,val in value.items():
        if field in result and result[field] not in val:
            raise Exception(f"{field} can only be in following format:\n{val}")
    try:
        int(result['plugin_version'])
    except ValueError as err:
        raise Exception(f"plugin_version must be an int:\n{value['plugin_version']}") from err
    attributes_list=[]
    for key,value in result.items():
        if key not in FUNCTIONAL_KEYS:
            attributes_list.append(key)
    if len(attributes_list) == 0:
        raise Exception("No attributes found")
    return True
